"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isPOJO = exports.isEventContainingNonFiniteNumericValues = exports.isEventPayloadTooBig = exports.EVENT_MAX_PAYLOAD = void 0;
/**
 * Maxium payload size for sending an event
 */
exports.EVENT_MAX_PAYLOAD = 16 * 1024;
/**
 * Calculate the size of the string in bytes and returns if it is to
 * big for the event payload
 *
 * @param str string which should be used for event payload
 * @returns true if the payload is too big
 */
const isEventPayloadTooBig = (str) => {
    let s = str.length;
    for (let i = str.length - 1; i >= 0; i--) {
        const code = str.charCodeAt(i);
        if (code > 0x7f && code <= 0x7ff) {
            s++;
        }
        else if (code > 0x7ff && code <= 0xffff) {
            s += 2;
        }
        if (code >= 0xdc00 && code <= 0xdfff) {
            i--; // trail surrogate
        }
    }
    return s > exports.EVENT_MAX_PAYLOAD;
};
exports.isEventPayloadTooBig = isEventPayloadTooBig;
/**
 * Goes through the attributes which are sent with the event and returns true
 * if NaN, -Inf or Inf is used.
 *
 * @param attributes Attributes sent via an event
 * @returns True if attributes contain NaN, -Inf or Inf
 */
const isEventContainingNonFiniteNumericValues = (attributes) => {
    for (const key in attributes) {
        if (Object.prototype.hasOwnProperty.call(attributes, key)) {
            if (isElementNonFiniteNumericValue(attributes[key])) {
                return true;
            }
        }
    }
    return false;
};
exports.isEventContainingNonFiniteNumericValues = isEventContainingNonFiniteNumericValues;
/**
 * @param obj Argument to be checked
 * @returns   True if passed in obj argument is a plain javascript object
 */
const isPOJO = (obj) => {
    if (obj === null || typeof obj !== 'object') {
        return false;
    }
    return Object.getPrototypeOf(obj) === Object.prototype;
};
exports.isPOJO = isPOJO;
/**
 * Goes through the JSON array and returns true if NaN, -Inf or Inf is used.
 *
 * @param array JSONArray which should be checked
 * @returns True if array contain NaN, -Inf or Inf
 */
const isArrayContainingNonFiniteNumericValues = (array) => {
    for (const item of array) {
        if (isElementNonFiniteNumericValue(item)) {
            return true;
        }
    }
    return false;
};
/**
 * Checks a JSONValue if NaN, -Inf or Inf is used.
 *
 * @param item JSONValue which should be checked
 * @returns True if item is/contains NaN, -Inf or Inf
 */
const isElementNonFiniteNumericValue = (item) => ((0, exports.isPOJO)(item) && (0, exports.isEventContainingNonFiniteNumericValues)(item))
    || (Array.isArray(item) && isArrayContainingNonFiniteNumericValues(item))
    || (typeof item === 'number' && !isFinite(item));
//# sourceMappingURL=EventPayloadUtils.js.map