"use strict";
/*
 * Copyright 2023 Dynatrace LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.EventPayload = void 0;
const TimestampProvider_1 = require("../provider/TimestampProvider");
const EventPayloadUtils_1 = require("../utils/EventPayloadUtils");
const TextEncoderUtil_1 = require("../utils/TextEncoderUtil");
const EventPayloadAttributes_1 = require("./EventPayloadAttributes");
class EventPayload {
    constructor(config, timestampProvider = TimestampProvider_1.defaultTimestampProvider) {
        this.config = config;
        this.timestampProvider = timestampProvider;
        this.logger = config.openKit.loggerFactory.createLogger('EventPayload');
    }
    getBizEventsPayload(type, attributes, session) {
        const internalAttributes = Object.assign({}, attributes);
        this.addNonOverridableAttribute(internalAttributes, 'event.type', type);
        const sizePayload = (0, TextEncoderUtil_1.lengthInUtf8Bytes)(JSON.stringify(internalAttributes));
        this.addBasicEventData(internalAttributes, session);
        this.addNonOverridableAttribute(internalAttributes, EventPayloadAttributes_1.EVENT_KIND, EventPayloadAttributes_1.EVENT_KIND_BIZ);
        this.addNonOverridableAttribute(internalAttributes, 'dt.rum.custom_attributes_size', sizePayload);
        if ((0, EventPayloadUtils_1.isEventContainingNonFiniteNumericValues)(internalAttributes)) {
            this.addNonOverridableAttribute(internalAttributes, 'dt.rum.has_nfn_values', true);
        }
        return JSON.stringify(internalAttributes);
    }
    getCustomEventsPayload(name, attributes, session) {
        const internalAttributes = Object.assign({}, attributes);
        this.addBasicEventData(internalAttributes, session);
        this.addNonOverridableAttribute(internalAttributes, 'event.name', name);
        this.addOverridableAttribute(internalAttributes, EventPayloadAttributes_1.EVENT_KIND, EventPayloadAttributes_1.EVENT_KIND_RUM);
        if ((0, EventPayloadUtils_1.isEventContainingNonFiniteNumericValues)(internalAttributes)) {
            this.addNonOverridableAttribute(internalAttributes, 'dt.rum.has_nfn_values', true);
        }
        return JSON.stringify(internalAttributes);
    }
    addBasicEventData(attributes, session) {
        this.removeReservedInternalAttributes(attributes);
        this.addOverridableAttribute(attributes, EventPayloadAttributes_1.TIMESTAMP, this.timestampProvider.getCurrentTimestampNs());
        this.addNonOverridableAttribute(attributes, 'dt.rum.schema_version', '1.2');
        this.addNonOverridableAttribute(attributes, 'dt.rum.application.id', this.config.openKit.applicationId);
        this.addNonOverridableAttribute(attributes, 'dt.rum.instance.id', this.config.openKit.deviceId);
        this.addNonOverridableAttribute(attributes, 'dt.rum.sid', session.toString());
        this.addOverridableAttribute(attributes, EventPayloadAttributes_1.APP_VERSION, this.config.meta.applicationVersion);
        this.addOverridableAttribute(attributes, EventPayloadAttributes_1.OS_NAME, this.config.meta.operatingSystem);
        this.addOverridableAttribute(attributes, EventPayloadAttributes_1.DEVICE_MANUFACTURER, this.config.device.manufacturer);
        this.addOverridableAttribute(attributes, EventPayloadAttributes_1.DEVICE_MODEL_IDENTIFIER, this.config.device.modelId);
        this.addOverridableAttribute(attributes, EventPayloadAttributes_1.EVENT_PROVIDER, this.config.openKit.applicationId);
    }
    removeReservedInternalAttributes(attributes) {
        const jsonKeys = Object.keys(attributes);
        for (const key of jsonKeys) {
            if (key === 'dt' || key.startsWith('dt.')) {
                this.logger.warn('getEventsPayload', 'key name dt or dt. is reserved. Data will be removed!', { key });
                delete attributes[key];
            }
        }
    }
    addOverridableAttribute(attributes, key, value) {
        if (value !== undefined) {
            if (!(key in attributes)) {
                attributes[key] = value;
            }
        }
    }
    addNonOverridableAttribute(attributes, key, value) {
        if (value !== undefined) {
            if (key in attributes) {
                this.logger.warn('addAttribute', `key ${key} is reserved. Custom data will be removed!`, { key });
            }
            attributes[key] = value;
        }
    }
}
exports.EventPayload = EventPayload;
//# sourceMappingURL=EventPayload.js.map