"use strict";
/*
 * Copyright 2019 Dynatrace LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.StaticPayloadBuilder = void 0;
const PlatformConstants_1 = require("../PlatformConstants");
const Payload_1 = require("./Payload");
const PayloadQueryBuilder_1 = require("./PayloadQueryBuilder");
/**
 * Maximum size of a stacktrace passed to Dynatrace.
 */
const MAX_STACKTRACE_LENGTH = 128000;
/**
 * Maximum size of a reason message passed to Dynatrace.
 */
const MAX_REASON_LENGTH = 1000;
class StaticPayloadBuilder {
    static reportCrash(errorName, reason, stacktrace, sequenceNumber, timeSinceSessionStart) {
        let stacktraceTruncateLength = MAX_STACKTRACE_LENGTH;
        // Truncating stacktrace at last line break
        if (stacktrace.length > MAX_STACKTRACE_LENGTH) {
            const lastLineBreakIndex = stacktrace.lastIndexOf('\n', MAX_STACKTRACE_LENGTH);
            if (lastLineBreakIndex !== -1) {
                stacktraceTruncateLength = lastLineBreakIndex;
            }
        }
        return StaticPayloadBuilder.basicEventData(50 /* EventType.Crash */, errorName)
            .add("pa" /* PayloadKey.ParentActionId */, 0)
            .add("s0" /* PayloadKey.StartSequenceNumber */, sequenceNumber)
            .add("t0" /* PayloadKey.Time0 */, timeSinceSessionStart)
            .add("rs" /* PayloadKey.Reason */, reason, MAX_REASON_LENGTH)
            .add("st" /* PayloadKey.Stacktrace */, stacktrace, stacktraceTruncateLength)
            .add("tt" /* PayloadKey.ErrorTechnologyType */, PlatformConstants_1.errorTechnologyType)
            .build();
    }
    static startSession(sequenceNumber) {
        return StaticPayloadBuilder.basicEventData(18 /* EventType.SessionStart */)
            .add("pa" /* PayloadKey.ParentActionId */, 0)
            .add("s0" /* PayloadKey.StartSequenceNumber */, sequenceNumber)
            .add("t0" /* PayloadKey.Time0 */, 0)
            .build();
    }
    static endSession(sequenceNumber, duration) {
        return StaticPayloadBuilder.basicEventData(19 /* EventType.SessionEnd */)
            .add("pa" /* PayloadKey.ParentActionId */, 0)
            .add("s0" /* PayloadKey.StartSequenceNumber */, sequenceNumber)
            .add("t0" /* PayloadKey.Time0 */, duration)
            .build();
    }
    static action(name, actionId, startSequenceNumber, endSequenceNumber, timeSinceSessionStart, duration) {
        return StaticPayloadBuilder.basicEventData(1 /* EventType.ManualAction */, name)
            .add("ca" /* PayloadKey.ActionId */, actionId)
            .add("pa" /* PayloadKey.ParentActionId */, 0)
            .add("s0" /* PayloadKey.StartSequenceNumber */, startSequenceNumber)
            .add("s1" /* PayloadKey.EndSequenceNumber */, endSequenceNumber)
            .add("t0" /* PayloadKey.Time0 */, timeSinceSessionStart)
            .add("t1" /* PayloadKey.Time1 */, duration)
            .build();
    }
    static reportNamedEvent(name, parentActionId, startSequenceNumber, timeSinceSessionStart) {
        return StaticPayloadBuilder.basicEventData(10 /* EventType.NamedEvent */, name)
            .add("pa" /* PayloadKey.ParentActionId */, parentActionId)
            .add("s0" /* PayloadKey.StartSequenceNumber */, startSequenceNumber)
            .add("t0" /* PayloadKey.Time0 */, timeSinceSessionStart)
            .build();
    }
    static sendEvent(jsonPayload) {
        return new PayloadQueryBuilder_1.PayloadQueryBuilder()
            .add("et" /* PayloadKey.EventType */, 98 /* EventType.Event */)
            .addWithoutTruncate("pl" /* PayloadKey.EventPayload */, jsonPayload)
            .build();
    }
    static applicationWidePrefix(config) {
        const { openKit, device, privacy, meta } = config;
        return new PayloadQueryBuilder_1.PayloadQueryBuilder()
            .add("vv" /* PayloadKey.ProtocolVersion */, PlatformConstants_1.protocolVersion)
            .add("va" /* PayloadKey.OpenKitVersion */, PlatformConstants_1.openKitVersion)
            .add("ap" /* PayloadKey.ApplicationId */, openKit.applicationId)
            .addIfDefined("vn" /* PayloadKey.ApplicationVersion */, meta.applicationVersion)
            .addIfDefined("os" /* PayloadKey.DeviceOs */, meta.operatingSystem)
            .add("pt" /* PayloadKey.PlatformType */, PlatformConstants_1.platformTypeOpenKit)
            .add("tt" /* PayloadKey.AgentTechnologyType */, PlatformConstants_1.agentTechnologyType)
            .add("vi" /* PayloadKey.VisitorId */, openKit.deviceId)
            .add("dl" /* PayloadKey.DataCollectionLevel */, privacy.dataCollectionLevel)
            .add("cl" /* PayloadKey.CrashReportingLevel */, privacy.crashReportingLevel)
            .addIfDefined("mf" /* PayloadKey.DeviceManufacturer */, device.manufacturer)
            .addIfDefined("md" /* PayloadKey.DeviceModel */, device.modelId)
            .addIfDefined("sw" /* PayloadKey.ScreenWidth */, device.screenWidth)
            .addIfDefined("sh" /* PayloadKey.ScreenHeight */, device.screenHeight)
            .addIfDefined("ul" /* PayloadKey.UserLanguage */, device.userLanguage)
            .addIfDefined("so" /* PayloadKey.Orientation */, device.orientation)
            .build();
    }
    static sessionPrefix(prefix, sessionId, clientIpAddress, sessionStartTime) {
        const sessionPrefix = new PayloadQueryBuilder_1.PayloadQueryBuilder()
            .add("sn" /* PayloadKey.SessionNumber */, sessionId)
            .add("ip" /* PayloadKey.ClientIpAddress */, clientIpAddress)
            .add("tv" /* PayloadKey.SessionStartTime */, sessionStartTime)
            .build();
        return (0, Payload_1.combinePayloads)(prefix, sessionPrefix);
    }
    static mutable(multiplicity, transmissionTime, supplementaryBasicData) {
        return new PayloadQueryBuilder_1.PayloadQueryBuilder()
            .add("mp" /* PayloadKey.Multiplicity */, multiplicity)
            .add("tx" /* PayloadKey.TransmissionTime */, transmissionTime)
            .addIfDefined("np" /* PayloadKey.NetworkTechnology */, supplementaryBasicData.networkTechnology)
            .addIfDefined("ct" /* PayloadKey.ConnectionType */, supplementaryBasicData.connectionType)
            .addIfDefined("cr" /* PayloadKey.Carrier */, supplementaryBasicData.carrier)
            .build();
    }
    static reportValue(actionId, name, value, sequenceNumber, timeSinceSessionStart) {
        const eventType = typeof value === 'number'
            ? 13 /* EventType.ValueDouble */
            : 11 /* EventType.ValueString */;
        return StaticPayloadBuilder.basicEventData(eventType, name)
            .add("pa" /* PayloadKey.ParentActionId */, actionId)
            .add("s0" /* PayloadKey.StartSequenceNumber */, sequenceNumber)
            .add("t0" /* PayloadKey.Time0 */, timeSinceSessionStart)
            .addIfDefinedAndNotNull("vl" /* PayloadKey.Value */, value)
            .build();
    }
    static reportError(name, parentActionId, startSequenceNumber, timeSinceSessionStart, errorValue) {
        return StaticPayloadBuilder.basicEventData(40 /* EventType.Error */, name)
            .add("pa" /* PayloadKey.ParentActionId */, parentActionId)
            .add("s0" /* PayloadKey.StartSequenceNumber */, startSequenceNumber)
            .add("t0" /* PayloadKey.Time0 */, timeSinceSessionStart)
            .add("ev" /* PayloadKey.ErrorValue */, errorValue)
            .add("tt" /* PayloadKey.ErrorTechnologyType */, PlatformConstants_1.errorTechnologyType)
            .build();
    }
    static identifyUser(userTag, sequenceNumber, timeSinceSessionStart) {
        return StaticPayloadBuilder.basicEventData(60 /* EventType.IdentifyUser */, userTag)
            .add("pa" /* PayloadKey.ParentActionId */, 0)
            .add("s0" /* PayloadKey.StartSequenceNumber */, sequenceNumber)
            .add("t0" /* PayloadKey.Time0 */, timeSinceSessionStart)
            .build();
    }
    static webRequest(url, parentActionId, startSequenceNumber, timeSinceSessionStart, endSequenceNumber, duration, bytesSent, bytesReceived, responseCode) {
        return StaticPayloadBuilder.basicEventData(30 /* EventType.WebRequest */, url)
            .add("pa" /* PayloadKey.ParentActionId */, parentActionId)
            .add("s0" /* PayloadKey.StartSequenceNumber */, startSequenceNumber)
            .add("t0" /* PayloadKey.Time0 */, timeSinceSessionStart)
            .add("s1" /* PayloadKey.EndSequenceNumber */, endSequenceNumber)
            .add("t1" /* PayloadKey.Time1 */, duration)
            .addIfNotNegative("bs" /* PayloadKey.BytesSent */, bytesSent)
            .addIfNotNegative("br" /* PayloadKey.BytesReceived */, bytesReceived)
            .addIfNotNegative("rc" /* PayloadKey.ResponseCode */, responseCode)
            .build();
    }
    static basicEventData(eventType, name) {
        return new PayloadQueryBuilder_1.PayloadQueryBuilder()
            .add("et" /* PayloadKey.EventType */, eventType)
            .addIfDefined("na" /* PayloadKey.KeyName */, name)
            .add("it" /* PayloadKey.ThreadId */, 1);
    }
}
exports.StaticPayloadBuilder = StaticPayloadBuilder;
//# sourceMappingURL=StaticPayloadBuilder.js.map